# -*- coding: utf-8 -*-
"""
Created on Mon Jan 17 08:04:58 2022

Uses bootstrap to resample data with replacement;
The bootstrap sample will be used to train a SVM and make feature selection;
    
@author: flavio
"""

import pandas as pd
from sklearn.impute import SimpleImputer
import numpy as np

from sklearn import svm
from sklearn.feature_selection import RFECV
from sklearn.utils import resample

num_samples = 1000
df_missing_report = pd.read_csv('./rs_class_report.csv',sep=';')

#Opens main file 
df = pd.read_csv('./processed_data/cleaned_data_v2.csv',sep=';')
df = df.drop(['ID'], axis=1)

#Colect list with 10% missing values
df_10percent = pd.read_excel('./rs_to_remove/snp_to_remove_threshold_10percent.xlsx',names=['RS'])

features_log = []

#Removes SNP according to list, keeping only SNP with less than 10% missing values
df_filtered = df.drop(df_10percent['RS'].values, axis=1)
    
rs_names = df_filtered.columns[1:]

x_values = df_filtered.loc[:, df_filtered.columns != 'Class'].values
y_values = list(df_filtered['Class'].values)
    
"""
Uses most common values within each feature to replace missing values
"""    
imp_mean = SimpleImputer(missing_values=np.nan, strategy='most_frequent')
imp_mean.fit(x_values)
x_values1 = imp_mean.transform(x_values)
x_values1 = [list(x) for x in x_values1]

for sample_number in range(num_samples):  
    """
    Creates sample
    """
    print("Sample nº %s of %s"%(sample_number+1,num_samples))
    x_sample = resample(x_values1, replace=True)
    x_test = [x for x in x_values1 if x not in x_sample]
    
    y_sample = []
    for x in x_sample:
        y_sample.append(y_values[x_values1.index(x)])
        
    y_test = []
    for x in x_test:
        y_test.append(y_values[x_values1.index(x)])
    
    
    #Makes feature selection with RFE-CV
    model_selector = svm.SVC(kernel='linear')
    selector = RFECV(model_selector,  step=1)
    selector = selector.fit(x_sample, y_sample)
    x_classification = selector.transform(x_sample)
    x_test= selector.transform(x_test)
    
    #Stores selectec features for counting them later
    for i in range(len(rs_names)):
        if selector.support_[i] == True:
            df_temp = df_missing_report[df_missing_report['rs'] == rs_names[i]]
            features_log.append([sample_number,df_temp['rs'].values[0],"%s missing c0/c1: %s / %s"%(df_temp['rs'].values[0],df_temp['missing_class_0'].values[0],df_temp['missing_class_1'].values[0])])
    
#Counts the number of times a given RS apeared
df_temp = pd.DataFrame(features_log,columns=["Sample ID","RS","Obs"])

s1 = df_temp['RS'].value_counts()
s1.to_csv('Exp_1000_resamplings_10percMissing.csv',sep=";")