 #-*- coding: utf-8 -*-
"""
Created on Thu Apr 28 10:24:18 2022

@author: flavio
"""


import pandas as pd

import numpy as np
from sklearn.metrics import accuracy_score
from sklearn.impute import SimpleImputer
from sklearn import svm
from sklearn.metrics import confusion_matrix


def print_statistics(y_test,y_predicted):
    accuracy = accuracy_score(y_test,y_predicted)
    
    cm = confusion_matrix(y_test,y_predicted) 
    specificity = cm[1,1]/(cm[1,0]+cm[1,1])         
    sensitivity = cm[0,0]/(cm[0,0]+cm[0,1])
    f1 = f1_score(y_test, y_predicted)
    auc = roc_auc_score(y_test, y_predicted)
    print("----------------------")
    print("Test acc = ",accuracy)
    print('Specificity =',specificity)
    print('Sensitivity =',sensitivity)
    print('F1 = ',f1)
    print("AUC = ",auc)
    print(cm)
    print("----------------------")


"""
1 - data cleaning according to feature_selection_experiment_v6.py
"""
df = pd.read_csv('./processed_data/cleaned_data_v2.csv',sep=';')
df = df.drop(['ID'],axis=1)

selected_features_10percent = ['rs1051922',
                    'rs12340866',
                    'rs16923189',
                    'rs17622656',
                    'rs17804441',
                    'rs1990760',
                    'rs2161525',
                    'rs2508450',
                    'rs3024498',
                    'rs303215',
                    'rs3771300',
                    'rs4251513',
                    'Class']
 

# result = []
# for x in total:
#     if x not in selected_features_10percent:
#         result.append(x)
        


features_names = ['IFNB1 - rs1051922 (G>A)',
                    'JAK2 / INSL6  - rs12340866 (G>A)',
                    'PDCD1LG2 - rs16923189 (A>G)',
                    'IRF1 - rs17622656 (G>A)',
                    'CD274 - rs17804441 (T>C)',
                    'IFIH1 - rs1990760 (C>T)',
                    'CLEC4M / DC-SIGNR  - rs2161525 (T>C)',
                    'IL10RA - rs2508450 (T>C)',
                    'IL10  - rs3024498 (T>C)',
                    'IFIT1 - rs303215 (T>C)',
                    'STAT1 - rs3771300 (T>G)',
                    'IRAK4 - rs4251513 (G>C)',]


df = df[selected_features_10percent]

"""
Separates input from output data
"""
x_values = df.loc[:, df.columns != 'Class'].values
y_values = df['Class'].values

"""
Uses features most common to replace missing values
"""    
imp_mean = SimpleImputer(missing_values=np.nan, strategy='most_frequent')
imp_mean.fit(x_values)
x_values1 = imp_mean.transform(x_values)


"""
2 - Separate Training and Test Sets
"""
from sklearn.model_selection import train_test_split
x_train, x_test, y_train, y_test = train_test_split(x_values1, y_values, stratify=y_values,test_size=0.2,random_state=0)


"""
3 - check performance over test set
"""

from sklearn.metrics import roc_auc_score,f1_score

mlp = svm.SVC(C=1,kernel='linear')
mlp.fit(x_train, y_train)
y_predicted = mlp.predict(x_test)

print("SVM")
print_statistics(y_test,y_predicted)

"""
Uses XAI to investigate
"""
import shap

# Fits the explainer
explainer = shap.Explainer(mlp.predict, x_test, feature_names=features_names)
# Calculates the SHAP values - It takes some time
shap_values = explainer(x_test)

#IMPORTANT Here the features are ordered from the highest to the lowest effect on the prediction. 
shap.plots.beeswarm(shap_values,max_display=20)